﻿module CFireball;
import std.math;

public class CFireball
{
	private int mStartX;
	private int mStartY;
	private float mAngle;
	private int mType;
	private int mDamage;
	private int mSpeed;
	private int mRadius;

	private float mTime = 0;
	private float mX;
	private float mY;
	private float mLastX;
	private float mLastY;

	public this(int aStartX, int aStartY, float aAngle, int aType, int aDamage, int aSpeed,
			int aRadius)
	{
		mStartX = aStartX;
		mStartY = aStartY;
		mAngle = aAngle;
		mType = aType;
		mDamage = aDamage;
		mSpeed = aSpeed;
		mRadius = aRadius;

		mX = mStartX;
		mY = mStartY;
		mLastX = mStartX;
		mLastY = mStartY;
	}



	public int GetType()
	{
		return mType;
	}



	public int GetDamage()
	{
		return mDamage;
	}



	public int GetRadius()
	{
		return mRadius;
	}



	public int GetX()
	{
		return cast(int)mX;
	}



	public int GetY()
	{
		return cast(int)mY;
	}



	public float GetAngle()
	{
		return mAngle;
	}



	public bool IsFirst()
	{
		return mTime == 0;
	}



	public bool Update()
	{
		if (++mTime == 500)
			return true;
		mLastX = mX;
		mLastY = mY;
		mX = mStartX + cos(mAngle) * mSpeed * mTime;
		mY = mStartY + sin(mAngle) * mSpeed * mTime;
		return false;
	}



	public bool Collide(int aX, int aY)
	{
		float X0 = mLastX - mX;
		float Y0 = mLastY - mY;
		Normalise(X0, Y0);
		float X1 = aX - mX;
		float Y1 = aY - mY;
		Normalise(X1, Y1);
		float DP = X0 * X1 + Y0 * Y1;
		if (DP <= 0)
			return Distance2(mX, mY, aX, aY) <= mRadius ^^ 2;
		X1 = aX - mLastX;
		Y1 = aY - mLastY;
		Normalise(X1, Y1);
		if (-X0 * X1 - Y0 * Y1 <= 0)
			return Distance2(mLastX, mLastY, aX, aY) <= mRadius ^^ 2;

		//OLength = sqrt(H.Length ^^ 2 - H.Length ^^ 2 * cos(angle between H and A) ^^ 2)
		float HLength2 = Distance2(mX, mY, aX, aY);
		return HLength2 - HLength2 * DP ^^ 2 <= mRadius ^^ 2;
	}



	private float Distance2(float aX0, float aY0, float aX1, float aY1)
	{
		return (aX1 - aX0) ^^ 2 + (aY1 - aY0) ^^ 2;
	}



	private void Normalise(ref float aX, ref float aY)
	{
		float Length = sqrt(aX ^^ 2 + aY ^^ 2);
		aX = aX / Length;
		aY = aY / Length;
	}
}